@@ -44,6 +44,8 @@ class BaseSettings(PydanticBaseSettings):
     log_level: str = 'INFO'
     origin: str = None
 
+    bcrypt_rounds: int = 14
+
     asgi_path: str = 'foxglove.asgi:app'
     # TODO src here should be replaced with '.main:app' then prefixed with the name of the directory settings is in
     app: str = 'src.main:app'
@@ -54,7 +56,6 @@ class BaseSettings(PydanticBaseSettings):
     patch_paths: List[str] = ['src.patches']
 
     sql_path: Path = 'models.sql'
-    template_dir: Optional[Path] = 'templates'
     pg_dsn: Optional[str] = pg_dsn_default
     # eg. the db already exists on heroku and never has to be created
     pg_db_exists = False
